use leptos::{ev::keydown, *};
use leptos_use::*;
use strsim::normalized_damerau_levenshtein;
use web_sys::{ScrollIntoViewOptions, ScrollLogicalPosition};

use crate::util::surrealdb::Participant;

fn sort_participants(participants: Vec<Participant>, search: String) -> Vec<Participant> {
    let mut filtered_sorted_list: Vec<(Participant, f64)> = participants
        .into_iter()
        .map(|participant| {
            (
                participant.clone(),
                normalized_damerau_levenshtein(
                    &participant.name.to_lowercase(),
                    &search.to_lowercase(),
                ),
            )
        })
        .collect();

    filtered_sorted_list.sort_by(|a, b| {
        let (_, sim_score_a) = a;
        let (_, sim_score_b) = b;
        sim_score_b
            .partial_cmp(sim_score_a)
            .unwrap_or(std::cmp::Ordering::Equal)
    });

    filtered_sorted_list
        .into_iter()
        .map(|(item, _)| item)
        .collect()
}

/// Navigation bar
#[component]
pub fn Add() -> impl IntoView {
    let websocket = expect_context::<crate::util::surrealdb::SurrealContext>();
    let participants = expect_context::<crate::util::surrealdb::ParticipantsContext>();

    let container_ref: NodeRef<html::Ul> = create_node_ref();

    let (name, set_name) = create_signal(String::from(""));
    let (error, set_error) = create_signal(String::from(""));
    let (selected, set_selected) = create_signal::<usize>(0);

    let participants_sorted =
        create_memo(move |_| sort_participants(participants.read.get(), name.get()));

    let _ = use_event_listener(use_document(), keydown, move |evt| {
        if evt.key() == "ArrowDown".to_string() {
            set_selected.update(|x| {
                let len = participants.read.get_untracked().len();
                if *x != len {
                    *x += 1;
                }
            });
        } else if evt.key() == "ArrowUp".to_string() {
            set_selected.update(|x| {
                if *x != 0 {
                    *x -= 1
                }
            });
        }

        let el: web_sys::Element = container_ref
            .get_untracked()
            .unwrap()
            .children()
            .item(selected.get_untracked().try_into().unwrap())
            .expect("No element found");

        el.scroll_into_view_with_scroll_into_view_options(
            &ScrollIntoViewOptions::new().block(ScrollLogicalPosition::Center),
        );
    });

    let on_submit = move |ev: leptos::ev::SubmitEvent| {
        ev.prevent_default();
        set_error.set(String::from(""));
        /*
        match websocket.add_person(name.get(), group.get()) {
            Ok(_) => set_name.set(String::from("")),
            Err(err) => set_error.set(err),
        }
        */
    };

    view! {
        <h1>"Tijd toevoegen"</h1>
        <form class="add" on:submit=on_submit>
            <input type="text"
                placeholder="Name"
                tabindex=0
                on:input=move |ev| {
                    set_name.set(event_target_value(&ev));
                    set_selected.set(0);
                }
                prop:value=name
            />
            <ul class="participants-container" tabindex=-1 node_ref=container_ref>
                {move || participants_sorted.get().into_iter().enumerate().map(|(i, participant)| view! {
                    <li on:click=move |_| set_selected.set(i) class:selected=move || selected.get() == i>{participant.name + " " + "(" + &participant.group[6..].to_string() + ")" }</li>
                }).collect_view()}
            </ul>
            <div class="time-input-container">
                <span>"Tijd:"</span>
                <input type="number" tabindex=0 placeholder="mm" min=0 max=99 />
                <input type="number" tabindex=0 placeholder="ss" min=0 max=59 />
                <input type="number" tabindex=0 placeholder="ms" min=0 max=99 />
            </div>
            <input type="submit" tabindex=0 value="Submit" />
        </form>
        <p class="error">
            { error }
        </p>
    }
}
